/* ControlPanel.c
 * 
 * Author: Ben Levitt
 * Last Updated: 7/9/2001
 * Version 1.2
 * 
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the license, or (at your
 * option) any later version.
 *      
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *      
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <Pilot.h>
#include "StarterRsc.h"
#include "Prefs.h"


Boolean PrefsFormHandleEvent(EventPtr eventP);

static void SetupForm(void);
static FieldPtr SetFieldTextFromHandle(Word fieldID, VoidHand txtH);
static FieldPtr SetFieldTextFromStr(Word fieldID, CharPtr strP);
static void IntToString(int n, char *buf);
static int StringToInt(char *buf);
static VoidPtr GetObjectPtr(Word objectID);



Boolean PrefsFormHandleEvent(EventPtr eventP)
{
	MenuHackPreferenceType prefs;
    Boolean handled = false;
	char *str;
	
	switch (eventP->eType) 
		{
		case ctlSelectEvent:
			switch(eventP->data.ctlSelect.controlID)
				{
				case ControlDoneButton:
					prefs = GetPrefs();
					
					str = FldGetTextPtr(GetObjectPtr(ControlDragField));
					if (str && str[0]!=0)
						prefs.drag = StringToInt(str);
					if (prefs.drag < 1) prefs.drag = 1;
					if (prefs.drag > 160) prefs.drag = 160;
					
					str = FldGetTextPtr(GetObjectPtr(ControlDelayField));
					if (str && str[0]!=0)
						prefs.delay = StringToInt(str);
					if (prefs.delay < 1) prefs.delay = 1;
					if (prefs.delay > 120) prefs.delay = 120;
					
					str = FldGetTextPtr(GetObjectPtr(ControlCornerField));
					if (str && str[0]!=0)
						prefs.width = StringToInt(str);
					if (prefs.width > 160) prefs.width = 160;
					
					prefs.betterMenus = CtlGetValue(GetObjectPtr(ControlMenusCheckbox));
					prefs.betterPopups = CtlGetValue(GetObjectPtr(ControlPopupsCheckbox));
					
					// Write the saved preferences information.  This data 
					// will be backed up during a HotSync.
					PrefSetAppPreferences (fileCreator, prefID, prefVersionNum, 
						&prefs, sizeof (prefs), true);
					
					FrmGotoForm(9000);
					handled = true;
					break;
					
				case ControlDefaultsButton:
					PrefSetAppPreferences (fileCreator, prefID, prefVersionNum, 
						NULL, 0, true);
					SetupForm();
					handled = true;
					break;
			}
			break;
			
		case frmOpenEvent:
			SetupForm();
			handled = true;
			break;

		default:
			break;
		
		}
	
	return handled;
}


static void SetupForm(void)
{
	MenuHackPreferenceType prefs;
    FormPtr frmP;
	char buf[4];

	frmP = FrmGetActiveForm();
	prefs = GetPrefs();
	
	IntToString(prefs.drag, buf);
	SetFieldTextFromStr(ControlDragField, buf);
	IntToString(prefs.delay, buf);
	SetFieldTextFromStr(ControlDelayField, buf);
	IntToString(prefs.width, buf);
	SetFieldTextFromStr(ControlCornerField, buf);
	
	CtlSetValue(GetObjectPtr(ControlMenusCheckbox), prefs.betterMenus);
	CtlSetValue(GetObjectPtr(ControlPopupsCheckbox), prefs.betterPopups);
	
	FrmDrawForm ( frmP);
}


// Used to set Field text in preferences
static FieldPtr SetFieldTextFromHandle(Word fieldID, VoidHand txtH)
{
	VoidHand	oldTxtH;
	FormPtr	frm = FrmGetActiveForm();
	FieldPtr	fldP;
	
	// get the field and the field's current text VoidHand.
	fldP = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, fieldID));
	ErrNonFatalDisplayIf(!fldP, "missing field");
	oldTxtH = (VoidHand) FldGetTextHandle(fldP);
	
	// set the field's text to the new text
	FldSetTextHandle(fldP, (Handle) txtH);
	FldDrawField(fldP);
	
	// free the VoidHand AFTER we call FldSetTextHandle.
	if (oldTxtH)
		MemHandleFree(oldTxtH);
	
	return fldP;
}


// Allocates new VoidHand and copies incoming string
static FieldPtr SetFieldTextFromStr(Word fieldID, CharPtr strP)
{
	VoidHand txtH;
	
	// get some space in which to stash the string
	txtH = MemHandleNew(StrLen(strP) + 1);
	if (!txtH)
		return NULL;
		
	// copy the string to the locked VoidHand.
	StrCopy(MemHandleLock(txtH), strP);
	
	// unlock the string VoidHand 
	MemHandleUnlock(txtH);
	
	// set the field to the VoidHand
	return SetFieldTextFromHandle(fieldID, txtH);
}


// For numbers 0 - 999 inclusive
static void IntToString(int n, char *buf)
{
	if(n > 99) {
		buf[0] = '0' + (n / 100);
		buf[1] = '0' + (n / 10) % 10;
		buf[2] = '0' + n % 10;
		buf[3] = 0;
	}
	else if (n > 9){
		buf[0] = '0' + n / 10;
		buf[1] = '0' + n % 10;
		buf[2] = 0;
	}
	else {
		buf[0] = '0' + n;
		buf[1] = 0;
	}
}


static int StringToInt(char *buf)
{
	int i, n;
	
	n=0;
	i=-1;
	while(buf[++i])
		n = 10*n + ((buf[i] >= '0' && buf[i] <= '9') ? (buf[i] - '0') : 0);
	
	return n;
}


static VoidPtr GetObjectPtr(Word objectID)
	{
	FormPtr frmP;


	frmP = FrmGetActiveForm();
	return (FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID)));
}






